/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005-2006 EBM WebSourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: FrontEngine.java,v 1.1 2007/06/11 13:34:07 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.se.rmi;

import java.io.File;
import java.net.MalformedURLException;
import java.rmi.AlreadyBoundException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;

import org.objectweb.petals.tools.rmi.server.remote.implementations.RemoteComponentContextImpl;
import org.objectweb.petals.tools.rmi.server.remote.implementations.RemoteDeliveryChannelImpl;
import org.objectweb.petals.tools.rmi.server.remote.interfaces.RemoteComponentContext;
import org.objectweb.petals.tools.rmi.server.remote.interfaces.RemoteDeliveryChannel;
import org.ow2.petals.se.rmi.registry.Registry;
import org.ow2.petals.se.rmi.registry.RmiProperties;
import org.ow2.petals.se.rmi.registry.RmiPropertiesException;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;

/**
 * Class FrontEngine.java.
 * 
 * @author Nicolas Salatge - EBM Websourcing
 * @author Roland NAUDIN - EBM Websourcing
 */
public class RmiComponent implements Component, ComponentLifeCycle {

    private static final String JBI_DESCRIPTOR_PATH = "META-INF/jbi.xml";

    /**
     * the RMI registry Server.
     */
    private Registry rmiServer;

    /**
     * the RMI component context.
     */
    private RemoteComponentContextImpl rmiComponentContext = null;

    /**
     * the concrete component context.
     */
    private javax.jbi.component.ComponentContext concreteComponentContext = null;

    /**
     * the RMI delivery channel.
     */
    private RemoteDeliveryChannel rmiDeliveryChannel = null;

    /**
     * the concrete delivery channel.
     */
    private javax.jbi.messaging.DeliveryChannel concreteDeliveryChannel = null;

    /**
     * the component logger.
     */
    private Logger logger;

    /**
     * HasMap containing all the messages exchanges.
     */
    private Map<String, MessageExchange> messages;

    /**
     * The RMI properties.
     */
    private RmiProperties rmiProperties = null;

    /**
     * Default Constructor.
     */
    public RmiComponent() {
    }

    /**
     * get the component logger.
     * 
     * @return the component logger
     */
    public Logger getLogger() {
        return this.logger;
    }

    /**
     * get messaging.
     * 
     * @return the messages
     */
    public Map<String, MessageExchange> getMessages() {
        return this.messages;
    }

    /**
     * start the component.
     * 
     * @throws JBIException
     *             impossible to start the component.
     */
    public void start() throws JBIException {
        this.logger.info("Starting RMI Service Engine...");
        this.logger.info(" - RMI Port : " + this.rmiProperties.getRmiPort());
        this.logger
                .info(" - Embedded : " + this.rmiProperties.isEmbeddedRegistry());
        this.logger.info(" - Component Context Name : "
                + this.rmiProperties.getComponentContextName());

        try {

            this.messages = new ConcurrentHashMap<String, MessageExchange>();

            this.rmiDeliveryChannel = new RemoteDeliveryChannelImpl(this.getChannel(), this
                    .getContext(), this.getMessages(), this.getLogger());

            this.rmiComponentContext = new RemoteComponentContextImpl(this.getContext(), this
                    .getRmiDeliveryChannel());

            this.rmiServer = new Registry(this.rmiProperties);

            this.rmiServer.initializeRegistry();

            this.rmiServer.registerRMIInterface(this.rmiComponentContext);

        } catch (RemoteException e) {
            this.getLogger().severe(e.getClass().getSimpleName() + " - " + e.getMessage());
            throw new JBIException(e);
        } catch (MalformedURLException e) {
            this.getLogger().severe(e.getClass().getSimpleName() + " - " + e.getMessage());
            throw new JBIException(e);
        } catch (AlreadyBoundException e) {
            this.getLogger().severe(e.getClass().getSimpleName() + " - " + e.getMessage());
            throw new JBIException(e);
        }

    }

    /**
     * Get the RMI component context.
     * 
     * @return the RMI component context
     */
    public RemoteComponentContext getRmiComponentContext() {
        return this.rmiComponentContext;
    }

    /**
     * get the rmi delivery channel.
     * 
     * @return the rmi delivery channel
     */
    public RemoteDeliveryChannel getRmiDeliveryChannel() {
        return this.rmiDeliveryChannel;
    }

    /**
     * stop the component.
     * 
     * @throws JBIException
     *             impossible to start the component.
     */
    public void stop() throws JBIException {
        this.logger.info("Stopping Service Engine...");
        try {
            this.rmiServer.deregisterRMIInterface();
        } catch (RemoteException e) {
            this.getLogger().log(Level.WARNING, e.getMessage(), e);
        } catch (MalformedURLException e) {
            this.getLogger().log(Level.SEVERE, e.getMessage(), e);
            throw new JBIException(e.getMessage());
        } catch (NotBoundException e) {
            this.getLogger().log(Level.SEVERE, e.getMessage(), e);
            throw new JBIException(e.getMessage());
        }
    }

    /**
     * Get the description of service.
     * 
     * @param endpoint
     *            the endpoint
     * @return the description
     */
    public Document getServiceDescription(final ServiceEndpoint endpoint) {
        return this.rmiComponentContext.getOwnedEndpointDescriptor(endpoint.getEndpointName());
    }

    /**
     * Get the component life cycle.
     * 
     * @return the component life cylce
     */
    public ComponentLifeCycle getLifeCycle() {
        return this;
    }

    /**
     * Get the service unit manager.
     * 
     * @return the service unit manager
     */
    public ServiceUnitManager getServiceUnitManager() {
        return null;
    }

    /**
     * Test if exchange with consumer is ok.
     * 
     * @param endpoint
     *            the endpoint
     * @param exchange
     *            the exchange
     * @return true if ok
     */
    public boolean isExchangeWithConsumerOkay(final ServiceEndpoint endpoint,
            final MessageExchange exchange) {
        return true;
    }

    /**
     * Test if exchange with provider is ok.
     * 
     * @param endpoint
     *            the endpoint
     * @param exchange
     *            the exchange
     * @return true if ok
     */
    public boolean isExchangeWithProviderOkay(final ServiceEndpoint endpoint,
            final MessageExchange exchange) {
        return true;
    }

    /**
     * Resolve the endpoint reference.
     * 
     * @param epr
     *            the document fragment
     * @return the service endpoint
     */
    public ServiceEndpoint resolveEndpointReference(final DocumentFragment epr) {
        return null;
    }

    /**
     * Get the extension MBean name.
     * 
     * @return the extension Mbean name
     */
    public ObjectName getExtensionMBeanName() {
        return null;
    }

    /**
     * Init the component.
     * 
     * @param context
     *            the context
     * @throws JBIException
     *             impossible to init the component
     */
    public void init(final javax.jbi.component.ComponentContext context) throws JBIException {
        this.concreteComponentContext = context;
        this.concreteDeliveryChannel = this.concreteComponentContext.getDeliveryChannel();
        this.logger = this.concreteComponentContext.getLogger("", null);

        if (this.logger == null) {
            throw new JBIException("The logger is null");
        }
        this.logger.log(Level.INFO, "INIT RMI engine");

        File jbiDescriptorFile = new File(this.getContext().getInstallRoot() + File.separator
                + JBI_DESCRIPTOR_PATH);
        try {
            this.rmiProperties = new RmiProperties(jbiDescriptorFile);
        } catch (RmiPropertiesException e) {
            throw new JBIException(
                    "Failed to retrieve the RMI properties from the component descriptor file", e);
        }
    }

    /**
     * get the context.
     * 
     * @return the component context
     */
    public javax.jbi.component.ComponentContext getContext() {
        return this.concreteComponentContext;
    }

    /**
     * get the delivery channel.
     * 
     * @return the delivery channel
     */
    public javax.jbi.messaging.DeliveryChannel getChannel() {
        return this.concreteDeliveryChannel;
    }

    /**
     * get the shutdown.
     * 
     * @throws JBIException
     *             impossible to realize the shutdown
     */
    public void shutDown() throws JBIException {
        this.logger.log(Level.INFO, "shut down");
    }
}
