/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.se.rmi.registry;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * The RMI component properties.
 * 
 * @author Nicolas SALATGE - eBMWebSourcing
 * @author Roland NAUDIN - eBMWebSourcing
 * 
 */
public class RmiProperties {

    /**
     * RMI schema
     */
    private static final String RMI_SCHEMA_RESSOURCE = "/petalsRMI.xsd";

    private static final String RMI_EXTENSIONS_SCHEMA_RESSOURCE = "/petalsRMIExtensions.xsd";

    /**
     * XSD schema
     */
    private final String NAMESPACE_XSD = "http://www.w3.org/2001/XMLSchema";

    /**
     * the RMI host property.
     */
    public static final String RMI_HOST = "host";

    /**
     * the RMI port property.
     */
    public static final String RMI_PORT = "port";

    /**
     * Property: Verify if the internal registry server have to be activated.
     */
    public static final String RMI_EMBEDDED_REGISTRY = "embeddedregistry";

    /**
     * Property: the name of the RMI component context.
     */
    public static final String RMI_COMPONENT_CONTEXT_NAME = "componentcontextname";

    /**
     * the default RMI host.
     */
    public static final String DEFAULT_RMI_HOST = "localhost";

    /**
     * the default RMI port.
     */
    public static final int DEFAULT_RMI_PORT = 1099;

    /**
     * the internal registry server is active.
     */
    public static final boolean DEFAULT_RMI_EMBEDDED_REGISTRY = true;

    /**
     * the name of the RMI component context.
     */
    public static final String DEFAULT_RMI_COMPONENT_CONTEXT_NAME = "RMIComponentContext";

    /**
     * Private fields
     */
    private File jbiDescriptor;

    private String rmiHost;

    private int rmiPort;

    private boolean isEmbeddedRegistry;

    private String componentContextName;

    /**
     * Creates a new instance of {@link RmiProperties}.
     * 
     * @param propertiesManager
     *            the properties
     * @throws RmiPropertiesException
     *             Failed to retrieve the properties
     */
    public RmiProperties(final File jbiDescriptor) throws RmiPropertiesException {
        this.jbiDescriptor = jbiDescriptor;
        this.rmiHost = DEFAULT_RMI_HOST;
        this.rmiPort = DEFAULT_RMI_PORT;
        this.isEmbeddedRegistry = DEFAULT_RMI_EMBEDDED_REGISTRY;
        this.componentContextName = DEFAULT_RMI_COMPONENT_CONTEXT_NAME;

        parseJbi();
    }
    
    /**
     * Get the RMI port.
     * 
     * @return the RMI host
     */
    public String getRmiHost() {
        return this.rmiHost;
    }
    
    /**
     * Get the RMI port.
     * 
     * @return the RMI port
     */
    public int getRmiPort() {
        return this.rmiPort;
    }

    /**
     * Does the component have to start its internal RMI registry server.
     * 
     * @return true if the component have to start its internal RMI registry
     *         server
     */
    public boolean isEmbeddedRegistry() {
        return isEmbeddedRegistry;
    }

    public String getComponentContextName() {
        return componentContextName;
    }

    /**
     * Parse the JBI descriptor.
     */
    private void parseJbi() throws RmiPropertiesException {
        Document rmiDocument = getDomDocument();
        NodeList rmiNodeList = rmiDocument.getElementsByTagNameNS(
                "http://petals.ow2.org/extensions/rmi", "rmi");
        NodeList propertiesNodeList = rmiNodeList.item(0).getChildNodes();
        for (int i = 0; i < propertiesNodeList.getLength(); i++) {
            if (RMI_HOST.equals(propertiesNodeList.item(i).getLocalName())) {
                rmiHost = propertiesNodeList.item(i).getTextContent();
            } else if (RMI_PORT.equals(propertiesNodeList.item(i).getLocalName())) {
                rmiPort = Integer.valueOf(propertiesNodeList.item(i).getTextContent());
            } else if (RMI_EMBEDDED_REGISTRY.equals(propertiesNodeList.item(i).getLocalName())) {
                isEmbeddedRegistry = Boolean.valueOf(propertiesNodeList.item(i).getTextContent());
            } else if (RMI_COMPONENT_CONTEXT_NAME.equals(propertiesNodeList.item(i).getLocalName())) {
                componentContextName = propertiesNodeList.item(i).getTextContent();
            }
        }
    }

    private Document getDomDocument() throws RmiPropertiesException {
        Document document = null;

        try {
            final DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
            docBuilderFactory.setNamespaceAware(true);

            final SchemaFactory factory = SchemaFactory.newInstance(NAMESPACE_XSD);
            final URL schemaURL = this.getClass().getResource(RMI_SCHEMA_RESSOURCE);
            final URL schemaExtensionsURL = this.getClass().getResource(
                    RMI_EXTENSIONS_SCHEMA_RESSOURCE);
            Source[] schemas = new Source[] { new StreamSource(schemaExtensionsURL.openStream()),
                    new StreamSource(schemaURL.openStream()) };
            final Schema rmiSchema = factory.newSchema(schemas);
            docBuilderFactory.setSchema(rmiSchema);

            final DocumentBuilder docbuilder = docBuilderFactory.newDocumentBuilder();
            document = docbuilder.parse(this.jbiDescriptor);
        } catch (IOException e) {
            throw new RmiPropertiesException(e);
        } catch (SAXException e) {
            throw new RmiPropertiesException(e);
        } catch (ParserConfigurationException e) {
            throw new RmiPropertiesException(e);
        }

        return document;
    }

}
